/* Project : Test programm for POVS1 board (rev 1.2.1)
 * File:   main.c
 * Author: Philippe Arlotto
 *
 * Created on 4 juin 2012, 11:26
 */


#include "povS1.h"
#include "bluetooth.h"
#include "temp_data.h"

#pragma config OSC = HSPLL  // Fosc = Fquartz x 4
#pragma config WDT = OFF
#pragma config LVP = OFF
#pragma config PBADEN = OFF

void init ( void ) ;
void ClearAllLeds ( void ) ;
void SetAllLeds(void ) ;
void Delay1SECx( unsigned int nsec ) ;
void DecToLeds( unsigned char value) ;
void BinToLeds( unsigned int value);
unsigned char ReadSwitches(void);
unsigned int ReadVBAT(void) ;
int ReadTemp(void) ;
void LedOnx(unsigned char n);
void SwSetTime(void) ;

unsigned int t = 0 ;  // Compteur de tour de tche de fond
unsigned char sw ; // tat des switch (0-3)
unsigned int vbat ; // tension batterie en mV
volatile unsigned char LowSpeed = 1 ; // Indique vitesse trop basse pour POV
volatile unsigned int LedsArray[60] = {0}; // Etat des leds tous les 6
unsigned char x = 0 , sec0 ;
int temp ;
volatile char msg[MAX_MSG+1];
volatile unsigned char rx = 0 ;

extern unsigned char hour,min,sec ; // heure mise  jour dans interrup.c
extern const rom unsigned int temp_look_up_table[] ;

void main(void) {

    init();

    // Autorise interruptions
    RCONbits.IPEN = 1; // priorits actives
    INTCONbits.GIEH = 1 ;
    INTCONbits.GIEL = 1 ;
    sw = ReadSwitches();
    DecToLeds(sw);
    while (ReadSwitches());
    printf(VERSION"\r\n$");
    while (1)
    {

       // affiche temprature tant que BP2 appuy
       while (ReadSwitches()==2)  {
        temp = ReadTemp();
            temp -= 18 ; // on affiche  partir de 18C
            ClearAllLeds () ;
            if (temp>=0) {   
                LedOnx(temp/2); // 2C/LED
            }          
        }

        if (ReadSwitches()==3) {
             sec0 = sec ; // test entre mode rglage heure
            while(ReadSwitches()==3) {
                if (sec > (sec0+3)%60) { 
                    SetAllLeds();
                    SwSetTime();
                }
            }
        }
        if ( rx ) {
            LED8 = !LED8 ; // test rx
            execute_command();
            rx = 0 ;
        }
        // Si vitesse trop basse LED9 reflte tat du phototransistor
        if (LowSpeed) {
            LED9 = REF_INPUT ;
        }
        t++; 
        if ( t>60000) {
            t=0;
            // affiche vbat et led 11 clignotante tant que vitesse trop basse
            if (LowSpeed) {
                vbat = ReadVBAT();
                 LedOnx((vbat/1000)/2);
                 THERM_ON = 0 ; // pour ne pas chauffer la thermistance
                 LED11 = x;
                 x = 1-x;
            }
             
           
        }
    }
}


void init ( void ) {
    // Leds output
    TRISLED0to1  =  LED0to1_OUTPUT ;
    TRISLED2to7 =  LED2to7_OUTPUT ;
    TRISLED8to11 = LED8to11_OUTPUT ;
    // ADC : vref+ =5 , vref- = 0 , AN0-AN3 analogiques
    OpenADC(ADC_FOSC_64 &
        ADC_RIGHT_JUST &
        ADC_16_TAD  ,
        ADC_CH0 &
        ADC_INT_OFF &
        ADC_VREFPLUS_VDD &
        ADC_VREFMINUS_VSS,
        11 ); // AN0-AN3 analogs other digitals
     // Interruption REF active : index 0
    INTCON2bits.INTEDG0 = 0 ; // Interrupt on falling edge
    INTCONbits.INT0E = 1 ;
    // Timer 0 : mesure dure d'un tour :
    // Kdiv = 32 Toverflow = 32.2^16.0.1us = 209,72ms  40Mhz
    // 209,7ms/tour <=> 286 tr/min Overflow si V<286tr/min
    // Pour 500 tr/min = 120ms/tour N0timer =  37500
    OpenTimer0( TIMER_INT_ON  &
                T0_16BIT   &
                T0_SOURCE_INT &
                T0_PS_1_32
                 ) ;
    INTCON2bits.TMR0IP = 1 ; // timer 0 prio haute
    // Timer 3 : 120 it par tour soit tous les 3 (360/120)
    // A 500 tr/min 3 <=> 1ms
    // A 2000 tr/min 3 <=> 0.25ms
    // T3Overflow = 1x2^16x0.1us = 6,55ms
    OpenTimer3(TIMER_INT_ON &
               T3_16BIT_RW  &
               T3_SOURCE_INT &
               T3_PS_1_1    &
               T3_SYNC_EXT_OFF );
    IPR2bits.TMR3IP=1 ; // timer3 prio haute
    // Timer 1 : horloge
    //T1Overflow  = 8x2^16x0.1us = 52,24ms sans prchargement
    //50ms = 8xNx0.1u <=> N = 62500 prchargement  65536-62500 = 3036
     OpenTimer1(TIMER_INT_ON &
                T1_16BIT_RW  &
                T1_SOURCE_INT &
                T1_PS_1_8    &
                T1_SYNC_EXT_OFF &
                T1_OSC1EN_OFF );
     WriteTimer1(3036);
     IPR1bits.TMR1IP = 1 ; // timer1 prio haute
    // init 9600Bd  40Mhz avec it en rx
    OpenUSART( USART_TX_INT_OFF &
                USART_RX_INT_ON &
                USART_ASYNCH_MODE &
                USART_EIGHT_BIT &
                USART_CONT_RX &
                USART_BRGH_LOW,
                64 );
    IPR1bits.RCIP = 0 ; // uart prio basse
    SetAllLeds() ;
    Delay1SECx(3) ;
    ClearAllLeds () ;
    Delay1SECx(1) ;
}


void ClearAllLeds () {
     
    BinToLeds(0);
}

void SetAllLeds() {
  
    BinToLeds(4095);

}

void Delay1SECx(unsigned int nsec ) {

unsigned int n = (nsec *  SECOND_1OKTCY / 255) + 1 ;
while ( n--) {
  Delay10KTCYx( 255 ) ;
 }
}

// display a binary value on leds
void BinToLeds( unsigned int value) {

    LED0to1   =  (value & 0x0003) << 4 ;
    LED2to7  =   (value & 0x00FC) >> 2 ;
    LED8to11 =   (value & 0x0F00) >> 6 ;

}

// display a decimal value on leds
void DecToLeds(unsigned char value) {

    if ( value == 0 || value > 12) {
         BinToLeds(0);
         return ; }
  
    BinToLeds( (unsigned int)1<<(value-1) );

}

// Allume n leds
void LedOnx(unsigned char n) {
    unsigned char v=0 ;
    while(n--) {
        v <<= 1 ;
        v |=  1 ;
    }
    BinToLeds(v);
}
// Retourne 0  3 en fonction de l'tat des boutons
unsigned char ReadSwitches(void) {

    return 0x03 & (~(SW_PORT & 0X03)) ;

}
// Retourne VBAT en millivolts
unsigned int ReadVBAT(void) {

    unsigned int millivolts ;

    SetChanADC(CH_MVBAT);
    Delay1KTCYx( 2 ) ;
    ConvertADC();
    while( BusyADC() ) ;
    millivolts = ReadADC() * MVABT_RATIO ;
    millivolts = (5000L*millivolts)/1024 ;

    return millivolts ;

}

// Retourne la temprature en C
int ReadTemp(void) {
    unsigned int raw  , i;
    THERM_ON = 1 ;
    SetChanADC(CH_TEMP);
    Delay1KTCYx( 2 ) ;
    ConvertADC();
    while( BusyADC() ) ;
    THERM_ON = 0 ;
    raw = ReadADC();
    i=0;
    while (raw< temp_look_up_table[i] && i < TEMP_DATA_NB ) i++;
    return TEMP_DATA_MIN + i*TEMP_DATA_DELTA ;
}
void SwSetTime(void) {
   unsigned char done = 0 , state = 0 , btn ;
   unsigned char new_hour,new_min,new_sec ;
   unsigned char new_min_d , new_min_u ;
   Delay1SECx(1) ;
   ClearAllLeds () ;
   
   new_hour = hour ;
   new_min = min ;
   new_min_d = new_min / 10 ;
   new_min_u = new_min % 10 ;
   new_sec = 0 ;
   INTCONbits.GIEH = 0 ;
   while ( !done )
   {
       if ( ( btn=ReadSwitches()) ==3 ) done = 1 ;
       switch(state) {
           case 0 : // Rglage de l'heure
               DecToLeds(new_hour);
               LED11=LED_ON ;
               if ( btn==1 ) {
                   if ( new_hour < 11 ) { new_hour++; }
                   else new_hour=0;
                   state = 1 ;
               } else {
                   if (btn == 2) state = 22 ;
               }
               break ;
           case 1 :
               if ( btn == 0 ) state = 0 ;
               break ;
            case 22 :
               if ( btn == 0 ) state = 2 ;
               break ;
           case 2 : // Rglage des dizaines de minutes
               DecToLeds(new_min_d);
               LED11=LED_OFF ;
               LED10=LED_ON ;
               if ( btn==1 ) {
                   if ( new_min_d < 5 ) { new_min_d++; }
                   else new_min_d=0;
                   state = 3 ;
               } else {
                   if (btn == 2) state = 44 ;
               }
               break ;
           case 3 :
               if ( btn == 0 ) state = 2 ;
               break ;
           case 44 :
               if ( btn == 0 ) state = 4 ;
               break ;
           case 4 : // Rglage des units de minutes
               DecToLeds(new_min_u);
               LED11=LED_ON ;
               LED10=LED_ON ;
               if ( btn==1 ) {
                   if ( new_min_u < 9 ) { new_min_u++; }
                   else new_min_u=0;
                   state = 5 ;
               } else {
                   if (btn == 2) state = 66 ;
               }
               break ;
           case 5 :
               if ( btn == 0 ) state = 4 ;
               break ;
           case 66 :
               if ( btn == 0 ) state = 6 ;
               break ;
           case 6 : // Attente top dpart
               ClearAllLeds () ;
               BinToLeds(0X555);
               hour = new_hour ;
               min = 10 * new_min_d + new_min_u ;
               sec = 0 ;
               if ( btn == 1) {
               WriteTimer1(3036);
               INTCONbits.GIEH = 1 ;
               done =1 ;
               }
               break ;
           default : // erreur !!
               hour = min =sec = 0 ;
               WriteTimer1(3036);
               INTCONbits.GIEH = 1 ;
               done =1 ;
       }
    Delay10KTCYx( 100 ) ; // debounce delay
   }
}
